//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//Copyright 1999 Dharma Shukla.

//Keep in mind that I wrote this code just for fun :-) 
//So use it at your own risk! 

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

#pragma once
#ifndef _EVENTMGR_H_
#define _EVENTMGR_H_

#include "Base_Include\comobj.H"

//--------------------------------------------------------------------------------------------------------------
//Very basic event map 
//
//
//
//
//--------------------------------------------------------------------------------------------------------------
struct EventInfo
{
	DISPID				m_dispID;
	CComBSTR			m_strEventName;
};

//--------------------------------------------------------------------------------------------------------------
struct CEventMap :	public CComObjectRoot, 
					public IDispatch, 
					public DataHolder<IUnknown>
{
public:
	CEventMap():	m_pEventInfo(0),
					m_nEvents(0),
					m_dwMajor(0),
					m_dwMinor(0),
					m_libID(GUID_NULL),
					m_iidSrc(GUID_NULL),
					m_dwCookie(0xFFFFeeee)
	{}

	~CEventMap()
	{
		if(m_pEventInfo)
		{
			delete []m_pEventInfo;
			m_pEventInfo = 0;
		}
	}

BEGIN_COM_MAP(CEventMap)
	COM_INTERFACE_ENTRY(IDispatch)
END_COM_MAP()
	
	HRESULT FinalConstruct()
	{ 
		m_spObjUnk = m_pData;		
		Init();
		return S_OK;	
	}
	void FinalRelease(){ UnAdvise(); }

private:	
	//Initializes all the 3 members
	HRESULT GetEventTypeInfo(LPTYPEINFO *ppTypeInfo);
	HRESULT Init();
	
public:
	HRESULT Advise()
	{
		return AtlAdvise(m_spObjUnk.p,GetUnknown(),m_iidSrc,&m_dwCookie);
	}
	HRESULT UnAdvise()
	{
		if(m_dwCookie != 0xFFFFeeee)
		{
			HRESULT hr = AtlUnadvise(m_spObjUnk.p,m_iidSrc,m_dwCookie);
			m_dwCookie = 0xFFFFeeee;
			return hr;
		}
		return S_FALSE;
	}
	
	STDMETHOD(GetTypeInfoCount)(UINT* pctinfo)
	{
		if(!pctinfo) return E_POINTER;
	
		*pctinfo = 1;
		return S_OK;
	}

	STDMETHOD(GetTypeInfo)(UINT itinfo, LCID lcid, ITypeInfo** pptinfo)
	{
		if(!pptinfo) return E_POINTER;

		if(m_spTypeInfo.p)
		{
			*pptinfo = m_spTypeInfo.p;
			(*pptinfo)->AddRef();
			return S_OK;
		}
		return E_FAIL;
	}

	STDMETHOD(GetIDsOfNames)(REFIID riid, LPOLESTR* rgszNames, UINT cNames,LCID lcid, DISPID* rgdispid)
	{
		return m_spTypeInfo->GetIDsOfNames(rgszNames, cNames, rgdispid);
	}

	STDMETHOD(Invoke)(DISPID dispidMember, REFIID riid,LCID lcid, WORD wFlags, DISPPARAMS* pdispparams, VARIANT* pvarResult,EXCEPINFO* pexcepinfo, UINT* puArgErr);
	
public:	
	ULONG			GetNumberOfEvents() const	{ return m_nEvents;		}
	EventInfo*		GetEventInfo() const		{ return m_pEventInfo;	}


private:
	unsigned short		m_dwMajor;		//major version of typelib
	unsigned short		m_dwMinor;		//minor version of typelib
	GUID				m_libID;		//LIBID
	IID					m_iidSrc;		//IID of Event Source interafce
	ULONG				m_nEvents;		//# of event functions
	EventInfo			*m_pEventInfo;	//DISPID-FuncName array
	CComPtr<ITypeInfo>	m_spTypeInfo;	//TypeInfo of the Event IID
	CComPtr<IUnknown>	m_spObjUnk;		//Unk of the Obj
	DWORD				m_dwCookie;


};

#endif